/** 
 * @file gl860-bayer.c
 * @author Olivier LORIN, tiré du pilote Syntek par Nicolas VIVIEN
 * @date 2008-05-05
 * @version v0.9.0
 *
 * @brief Driver for GL860 USB video camera
 *
 * @note Copyright (C) Olivier LORIN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <media/v4l2-common.h>

#include "gl860.h"


#define MAX(a,b)	((a)>(b)?(a):(b))
#define MIN(a,b)	((a)<(b)?(a):(b))
#define CLIP(a,low,high) MAX((low),MIN((high),(a)))

void gl860_b2gris8(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);

void b2bgr24lattice(unsigned char *bayer, unsigned char *RGB, int IMG_HEIGHT, int IMG_WIDTH);

void gl860_b2rgb24(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);
void gl860_b2rgb32(uint8_t *, uint8_t *, 
		struct gl860_coord *, struct gl860_coord *, 
		const int, const int, const int);
void gl860_b2bgr24(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);
void gl860_b2bgr32(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);

void gl860_b2uyvy(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);
void gl860_b2yuyv(uint8_t *, uint8_t *,
		struct gl860_coord *, struct gl860_coord *,
		const int, const int, const int);

void redim(char unsigned *img, char unsigned *imgs, int lo, int ho, int lf, int hf, int nb, int vraiSXGA);


void gl860_LumCntrSat(uint8_t *img, const int width, const int height, int brightness, int contrast, int sat, int eqb, int teinte, int palette, int depth);

static int iLut[256];
static unsigned char lut[256], lutV[256], lutB[256];

static signed short gl860_yuv_interp[256][8] = {
	{0,0,0,0,0,0,0,0},{0,0,0,0,0,0,0,0},{0,1,0,0,0,1,0,0},{0,1,0,0,0,1,-1,0},
	{1,2,0,0,-1,2,-1,0},{1,2,0,0,-1,2,-2,0},{1,3,0,-1,-1,3,-2,0},{2,3,0,-1,-2,3,-2,0},
	{2,4,0,-1,-2,4,-3,0},{2,5,1,-1,-2,4,-3,0},{2,5,1,-1,-3,5,-4,0},{3,6,1,-1,-3,5,-4,0},
	{3,6,1,-2,-3,6,-5,0},{3,7,1,-2,-4,6,-5,-1},{4,7,1,-2,-4,7,-5,-1},{4,8,1,-2,-4,7,-6,-1},
	{4,9,1,-2,-5,8,-6,-1},{5,9,1,-2,-5,8,-7,-1},{5,10,2,-3,-5,9,-7,-1},{5,10,2,-3,-6,9,-7,-1},
	{5,11,2,-3,-6,10,-8,-1},{6,11,2,-3,-6,10,-8,-1},{6,12,2,-3,-7,11,-9,-1},{6,13,2,-3,-7,11,-9,-1},
	{7,13,2,-4,-7,12,-10,-1},{7,14,2,-4,-8,12,-10,-2},{7,14,2,-4,-8,13,-10,-2},{8,15,3,-4,-8,13,-11,-2},
	{8,15,3,-4,-9,14,-11,-2},{8,16,3,-4,-9,14,-12,-2},{8,17,3,-5,-9,15,-12,-2},{9,17,3,-5,-10,15,-12,-2},
	{9,18,3,-5,-10,16,-13,-2},{9,18,3,-5,-10,16,-13,-2},{10,19,3,-5,-11,17,-14,-2},{10,19,3,-5,-11,17,-14,-2},
	{10,20,4,-6,-11,18,-15,-2},{11,20,4,-6,-12,18,-15,-3},{11,21,4,-6,-12,19,-15,-3},{11,22,4,-6,-12,19,-16,-3},
	{11,22,4,-6,-13,20,-16,-3},{12,23,4,-6,-13,20,-17,-3},{12,23,4,-7,-13,21,-17,-3},{12,24,4,-7,-14,21,-18,-3},
	{13,24,5,-7,-14,22,-18,-3},{13,25,5,-7,-14,22,-18,-3},{13,26,5,-7,-15,23,-19,-3},{14,26,5,-7,-15,23,-19,-3},
	{14,27,5,-8,-15,24,-20,-3},{14,27,5,-8,-16,24,-20,-3},{14,28,5,-8,-16,25,-20,-4},{15,28,5,-8,-16,25,-21,-4},
	{15,29,5,-8,-17,26,-21,-4},{15,30,6,-8,-17,26,-22,-4},{16,30,6,-9,-17,27,-22,-4},{16,31,6,-9,-18,27,-23,-4},
	{16,31,6,-9,-18,28,-23,-4},{17,32,6,-9,-18,28,-23,-4},{17,32,6,-9,-19,29,-24,-4},{17,33,6,-9,-19,29,-24,-4},
	{17,34,6,-10,-19,30,-25,-4},{18,34,6,-10,-20,30,-25,-4},{18,35,7,-10,-20,31,-25,-5},{18,35,7,-10,-20,31,-26,-5},
	{19,36,7,-10,-21,32,-26,-5},{19,36,7,-10,-21,32,-27,-5},{19,37,7,-11,-21,33,-27,-5},{20,37,7,-11,-22,33,-28,-5},
	{20,38,7,-11,-22,34,-28,-5},{20,39,7,-11,-22,34,-28,-5},{20,39,7,-11,-23,35,-29,-5},{21,40,8,-11,-23,35,-29,-5},
	{21,40,8,-12,-23,36,-30,-5},{21,41,8,-12,-24,36,-30,-5},{22,41,8,-12,-24,37,-30,-6},{22,42,8,-12,-24,37,-31,-6},
	{22,43,8,-12,-25,38,-31,-6},{23,43,8,-12,-25,38,-32,-6},{23,44,8,-13,-25,39,-32,-6},{23,44,9,-13,-26,39,-33,-6},
	{23,45,9,-13,-26,40,-33,-6},{24,45,9,-13,-26,40,-33,-6},{24,46,9,-13,-27,41,-34,-6},{24,47,9,-14,-27,41,-34,-6},
	{25,47,9,-14,-27,42,-35,-6},{25,48,9,-14,-28,42,-35,-6},{25,48,9,-14,-28,43,-36,-6},{26,49,9,-14,-28,43,-36,-7},
	{26,49,10,-14,-29,44,-36,-7},{26,50,10,-15,-29,44,-37,-7},{26,51,10,-15,-29,45,-37,-7},{27,51,10,-15,-30,45,-38,-7},
	{27,52,10,-15,-30,46,-38,-7},{27,52,10,-15,-30,46,-38,-7},{28,53,10,-15,-31,47,-39,-7},{28,53,10,-16,-31,47,-39,-7},
	{28,54,10,-16,-31,48,-40,-7},{29,54,11,-16,-32,48,-40,-7},{29,55,11,-16,-32,49,-41,-7},{29,56,11,-16,-32,49,-41,-8},
	{29,56,11,-16,-33,50,-41,-8},{30,57,11,-17,-33,50,-42,-8},{30,57,11,-17,-33,51,-42,-8},{30,58,11,-17,-34,51,-43,-8},
	{31,58,11,-17,-34,52,-43,-8},{31,59,11,-17,-34,52,-43,-8},{31,60,12,-17,-35,53,-44,-8},{31,60,12,-18,-35,53,-44,-8},
	{32,61,12,-18,-35,54,-45,-8},{32,61,12,-18,-36,54,-45,-8},{32,62,12,-18,-36,55,-46,-8},{33,62,12,-18,-36,55,-46,-9},
	{33,63,12,-18,-37,56,-46,-9},{33,64,12,-19,-37,56,-47,-9},{34,64,12,-19,-37,57,-47,-9},{34,65,13,-19,-38,57,-48,-9},
	{34,65,13,-19,-38,58,-48,-9},{34,66,13,-19,-38,58,-48,-9},{35,66,13,-19,-39,59,-49,-9},{35,67,13,-20,-39,59,-49,-9},
	{35,68,13,-20,-39,60,-50,-9},{36,68,13,-20,-40,60,-50,-9},{36,69,13,-20,-40,61,-51,-9},{36,69,14,-20,-40,61,-51,-9},
	{37,70,14,-20,-41,62,-51,-10},{37,70,14,-21,-41,62,-52,-10},{37,71,14,-21,-41,63,-52,-10},{37,72,14,-21,-42,63,-53,-10},
	{38,72,14,-21,-42,64,-53,-10},{38,73,14,-21,-42,64,-54,-10},{38,73,14,-21,-43,65,-54,-10},{39,74,14,-22,-43,65,-54,-10},
	{39,74,15,-22,-43,66,-55,-10},{39,75,15,-22,-44,66,-55,-10},{40,75,15,-22,-44,67,-56,-10},{40,76,15,-22,-44,67,-56,-10},
	{40,77,15,-22,-45,68,-56,-11},{40,77,15,-23,-45,68,-57,-11},{41,78,15,-23,-45,69,-57,-11},{41,78,15,-23,-46,69,-58,-11},
	{41,79,15,-23,-46,70,-58,-11},{42,79,16,-23,-46,70,-59,-11},{42,80,16,-23,-47,71,-59,-11},{42,81,16,-24,-47,71,-59,-11},
	{43,81,16,-24,-47,72,-60,-11},{43,82,16,-24,-48,72,-60,-11},{43,82,16,-24,-48,73,-61,-11},{43,83,16,-24,-48,73,-61,-11},
	{44,83,16,-24,-49,74,-61,-12},{44,84,16,-25,-49,74,-62,-12},{44,85,17,-25,-49,75,-62,-12},{45,85,17,-25,-50,75,-63,-12},
	{45,86,17,-25,-50,76,-63,-12},{45,86,17,-25,-50,76,-64,-12},{46,87,17,-25,-51,77,-64,-12},{46,87,17,-26,-51,77,-64,-12},
	{46,88,17,-26,-51,78,-65,-12},{46,89,17,-26,-52,78,-65,-12},{47,89,18,-26,-52,79,-66,-12},{47,90,18,-26,-52,79,-66,-12},
	{47,90,18,-26,-53,80,-66,-13},{48,91,18,-27,-53,80,-67,-13},{48,91,18,-27,-53,81,-67,-13},{48,92,18,-27,-54,81,-68,-13},
	{49,92,18,-27,-54,82,-68,-13},{49,93,18,-27,-54,82,-69,-13},{49,94,18,-28,-54,83,-69,-13},{49,94,19,-28,-55,83,-69,-13},
	{50,95,19,-28,-55,84,-70,-13},{50,95,19,-28,-55,84,-70,-13},{50,96,19,-28,-56,85,-71,-13},{51,96,19,-28,-56,85,-71,-13},
	{51,97,19,-29,-56,86,-72,-13},{51,98,19,-29,-57,86,-72,-14},{52,98,19,-29,-57,87,-72,-14},{52,99,19,-29,-57,87,-73,-14},
	{52,99,20,-29,-58,88,-73,-14},{52,100,20,-29,-58,88,-74,-14},{53,100,20,-30,-58,89,-74,-14},{53,101,20,-30,-59,89,-74,-14},
	{53,102,20,-30,-59,90,-75,-14},{54,102,20,-30,-59,90,-75,-14},{54,103,20,-30,-60,91,-76,-14},{54,103,20,-30,-60,91,-76,-14},
	{55,104,20,-31,-60,92,-77,-14},{55,104,21,-31,-61,92,-77,-15},{55,105,21,-31,-61,93,-77,-15},{55,106,21,-31,-61,93,-78,-15},
	{56,106,21,-31,-62,94,-78,-15},{56,107,21,-31,-62,94,-79,-15},{56,107,21,-32,-62,95,-79,-15},{57,108,21,-32,-63,95,-79,-15},
	{57,108,21,-32,-63,96,-80,-15},{57,109,22,-32,-63,96,-80,-15},{58,109,22,-32,-64,97,-81,-15},{58,110,22,-32,-64,97,-81,-15},
	{58,111,22,-33,-64,98,-82,-15},{58,111,22,-33,-65,98,-82,-16},{59,112,22,-33,-65,99,-82,-16},{59,112,22,-33,-65,99,-83,-16},
	{59,113,22,-33,-66,100,-83,-16},{60,113,22,-33,-66,100,-84,-16},{60,114,23,-34,-66,101,-84,-16},{60,115,23,-34,-67,101,-84,-16},
	{60,115,23,-34,-67,102,-85,-16},{61,116,23,-34,-67,102,-85,-16},{61,116,23,-34,-68,103,-86,-16},{61,117,23,-34,-68,103,-86,-16},
	{62,117,23,-35,-68,104,-87,-16},{62,118,23,-35,-69,104,-87,-16},{62,119,23,-35,-69,105,-87,-17},{63,119,24,-35,-69,105,-88,-17},
	{63,120,24,-35,-70,106,-88,-17},{63,120,24,-35,-70,106,-89,-17},{63,121,24,-36,-70,107,-89,-17},{64,121,24,-36,-71,107,-90,-17},
	{64,122,24,-36,-71,108,-90,-17},{64,123,24,-36,-71,108,-90,-17},{65,123,24,-36,-72,109,-91,-17},{65,124,24,-36,-72,109,-91,-17},
	{65,124,25,-37,-72,110,-92,-17},{66,125,25,-37,-73,110,-92,-17},{66,125,25,-37,-73,111,-92,-18},{66,126,25,-37,-73,111,-93,-18},
	{66,127,25,-37,-74,112,-93,-18},{67,127,25,-37,-74,112,-94,-18},{67,128,25,-38,-74,113,-94,-18},{67,128,25,-38,-75,113,-95,-18},
	{68,129,25,-38,-75,114,-95,-18},{68,129,26,-38,-75,114,-95,-18},{68,130,26,-38,-76,115,-96,-18},{69,130,26,-38,-76,115,-96,-18},
	{69,131,26,-39,-76,116,-97,-18},{69,132,26,-39,-77,116,-97,-18},{69,132,26,-39,-77,117,-97,-19},{70,133,26,-39,-77,117,-98,-19},
	{70,133,26,-39,-78,118,-98,-19},{70,134,27,-39,-78,118,-99,-19},{71,134,27,-40,-78,119,-99,-19},{71,135,27,-40,-79,119,-100,-19},
	{71,136,27,-40,-79,120,-100,-19},{72,136,27,-40,-79,120,-100,-19},{72,137,27,-40,-80,121,-101,-19},{72,137,27,-40,-80,121,-101,-19},
	{72,138,27,-41,-80,122,-102,-19},{73,138,27,-41,-81,122,-102,-19},{73,139,28,-41,-81,123,-103,-19},{73,140,28,-41,-81,123,-103,-20},
	{74,140,28,-41,-82,124,-103,-20},{74,141,28,-42,-82,124,-104,-20},{74,141,28,-42,-82,125,-104,-20},{75,142,28,-42,-83,125,-105,-20},
	{75,142,28,-42,-83,126,-105,-20},{75,143,28,-42,-83,126,-105,-20},{75,144,28,-42,-84,127,-106,-20},{76,144,29,-43,-84,127,-106,-20}
};


/** 
 * @brief Decompress a frame
 *
 * This function permits to decompress a frame from the video stream.
 *
 * @param dev Device structure
 * 
 * @returns 0 if all is OK
 */
int gl860_decompress(struct usb_gl860 *dev)
{
	int factor;

	void *data;
	void *image;
	struct gl860_frame_buf *framebuf;
	struct gl860_coord imageb, imaget, imagef;

	if (dev == NULL)
		return -EFAULT;

	framebuf = dev->read_frame;

	if (framebuf == NULL)
		return -EFAULT;

	image  = dev->image_data;
	image += dev->images[dev->fill_image].offset;

	data = framebuf->data;

	

	switch (dev->resolution) {
		case WCAM_80x60:
			factor = 8;
			break;

		case WCAM_128x96:
			factor = 5;
			break;

		case WCAM_160x120:
			factor = 4;
			break;

		case WCAM_213x160:
			factor = 3;
			break;

		case WCAM_320x240:
			factor = 2;
			break;

		case WCAM_640x480:
			factor = 1;
			break;

		case WCAM_800x600:
			factor = 1;
			if (_0503A_ || _0503B_ || _0503C_) memset(data+598*800, 0, 2*800);
			break;

		case WCAM_1280x960:
			factor = 1;
			break;

		case WCAM_1600x1200:
			factor = 1;
			if (_0503A_ || _0503B_ || _0503C_) memset(data+1198*1600, 0, 2*1600);
			break;

		default:
			return -EFAULT;
	}

	imageb.x=dev->image.x;
	imageb.y=dev->image.y;
	imaget.x=dev->image.x/factor;
	imaget.y=dev->image.y/factor;
	imagef.x=dev->view.x;
	imagef.y=dev->view.y;

//	STK_INFO("PAL=%d [%d %d %d %d %d %d] +-v4l2=%d\n",dev->vsettings.palette,WCAM_PALETTE_RGB24,GL860_PALETTE_RGB32,GL860_PALETTE_BGR24,GL860_PALETTE_BGR32,GL860_PALETTE_UYVY,GL860_PALETTE_YUYV,dev->vsettings.v4l2); // Pas tout le monde demande le même !

	switch (dev->vsettings.palette) {
		case WCAM_PALETTE_GRIS:
//			gl860_b2gris8(data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2gris8(data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_RGB24:
//			gl860_b2rgb24(data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2rgb24(data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_RGB32:
//			gl860_b2rgb32(data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2rgb32(data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_BGR24:
//			gl860_b2bgr24(data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2bgr24(data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_BGR32:
//			gl860_b2bgr32(data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2bgr32(data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_UYVY:
//			gl860_b2uyvy( data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2uyvy( data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;

		case WCAM_PALETTE_YUYV:
//			gl860_b2yuyv( data, image, &dev->image, &dev->view, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			gl860_b2yuyv( data, image, &imageb, &imaget, dev->vsettings.hflip, dev->vsettings.vflip, factor);
			break;
	}

	redim(image, image, imaget.x, imaget.y, imagef.x, imagef.y, dev->vsettings.depth, (dev->image.x==1280 && dev->image.y==1024));
	if (_0503SIM_)
		gl860_LumCntrSat(image, imagef.x, imagef.y, -1, dev->vsettings.contrast, dev->vsettings.saturation, dev->vsettings.srclum, dev->vsettings.colour, dev->vsettings.palette, dev->vsettings.depth);
	if (_F191_)
		gl860_LumCntrSat(image, imagef.x, imagef.y, -1, dev->vsettings.contrast,                        -1, dev->vsettings.whiteness,                     0, dev->vsettings.palette, dev->vsettings.depth);
	if (_0503A_ || _0503B_ || _0503C_)
		gl860_LumCntrSat(image, imagef.x, imagef.y, -1,                      -1, dev->vsettings.saturation, dev->vsettings.srclum, dev->vsettings.colour, dev->vsettings.palette, dev->vsettings.depth);

	return 0;
}


void redim(char unsigned *img, char unsigned *imgs, int lo, int ho, int lf, int hf, int nb, int vraiSXGA)
{
	int x, y, dxg, dyb, dxd, dyh;
	int lref, href, lg;
	int nvxy[1600];
	char unsigned *pe, *ps;

	if (lf==lo && hf==ho) return;

	nb=nb/8;
	lg=nb*lf;		 // lg=((nb*lf-1)/4+1)*4;

	if (vraiSXGA)
	{
		if (lf>5*hf/4) { href=hf; lref=5*hf/4; } else { lref=lf; href=4*lref/5; }
	}
	else
	{
		if (lf>4*hf/3) { href=hf; lref=4*hf/3; } else { lref=lf; href=3*lref/4; }
	}
	dxg=(lf-lref)/2; dxd=lf-lref-dxg;
	dyb=(hf-href)/2; dyh=hf-href-dyb;

	for (x=0;x<1600;x++) nvxy[x]=nb*(x*lo/lref);

	// Plus proche voisin
   for (y=0;y<href;y++)
	{
		pe=img +nvxy[y]*lo;
		ps=imgs+y*lg;
		for (x=0;x<lref;x++,ps+=nb) memmove(ps, pe+nvxy[x], nb);
	}

	// Bordure noire horizontale
	if (lf!=lref)
		for (y=href-1;y>=0;y--)
		{
			memcpy(imgs, imgs+y*lg, nb*lref);
			memcpy(imgs+y*lg+nb*dxg, imgs, nb*lref);

			memset(imgs+lg*y, 0, nb*dxg);
			memset(imgs+lg*y+nb*(dxg+lref), 0, nb*dxd);
		}

	// Bordure noire horizontale
	if (hf!=href)
	{
		for (y=href-1;y>=0;y--) memcpy(imgs+(dyb+y)*lg, imgs+y*lg, nb*lref); 
		memset(imgs, 0, dyb*lg);
		memset(imgs+(dyb+href)*lg, 0, dyh*lg);
	}

}


void gl860_LumCntrSat(uint8_t *img, const int width, const int height, int brightness, int contrast, int sat, int eqb, int teinte, int palette, int depth)
{
	int i,x,v, neg=0, inv=0;
	unsigned char *p, *pLutR, *pLutV, *pLutB;
	short Y,R,V,B;
	char tmp;
	int tabV[]={255, 243, 221, 198, 182};
	int tabB[]={255, 238, 185, 136, 106};

	for (i=0;i<256;i++) { x=i; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
	v = (brightness - 32768) / 256;
	if (brightness!=-1) for (i=0;i<256;i++) { x = iLut[i]+v; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
	v = (contrast*100 / (256 * 12) - 200) / 4 + 100; // -2/4 ->23/4
	if (contrast!=-1) for (i=0;i<256;i++) { x = 128 + v*(iLut[i]-128)/100; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
	for (i=0;i<256;i++) lut[i]=(unsigned char)iLut[i];

	if (palette==WCAM_PALETTE_RGB24 || palette==WCAM_PALETTE_RGB32) { pLutB=lutB; pLutV=lutV; pLutR=lut; }
	if (palette==WCAM_PALETTE_BGR24 || palette==WCAM_PALETTE_BGR32) { pLutB=lut;  pLutV=lutV; pLutR=lutB; }
	if (eqb==0 || palette==WCAM_PALETTE_YUYV || palette==WCAM_PALETTE_UYVY) pLutV=pLutB=pLutR=lut;

	if (eqb!=0)
	{
		eqb=eqb/(256*52);

		for (i=0;i<256;i++) { x=i*255/tabV[eqb]; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		v = (brightness - 32768) / 256;
		if (brightness!=-1) for (i=0;i<256;i++) { x = iLut[i]+v; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		v = (contrast*100 / (256 * 12) - 200) / 4 + 100; // -2/4 ->23/4
		if (contrast!=-1) for (i=0;i<256;i++) { x = 128 + v*(iLut[i]-128)/100; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		for (i=0;i<256;i++) lutV[i]=(unsigned char)iLut[i];

		for (i=0;i<256;i++) { x=i*255/tabB[eqb]; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		v = (brightness - 32768) / 256;
		if (brightness!=-1) for (i=0;i<256;i++) { x = iLut[i]+v; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		v = (contrast*100 / (256 * 12) - 200) / 4 + 100; // -2/4 ->23/4
		if (contrast!=-1) for (i=0;i<256;i++) { x = 128 + v*(iLut[i]-128)/100; if (x<0) iLut[i]=0; else if (x>255) iLut[i]=255; else iLut[i]=x; }
		for (i=0;i<256;i++) lutB[i]=(unsigned char)iLut[i];
	}

	if (teinte<  0xFFFF/4);
	else if (teinte<  0xFFFF/2) sat=0;
	else if (teinte<3*0xFFFF/4) neg=1;
	else if (teinte<  0xFFFF  ) inv=1; 

	v = sat / (256 * 25);

//STK_INFO("BB %d %d\n",pLutB[200],lutB[200]);

	switch (palette) {
		case WCAM_PALETTE_RGB24:
		case WCAM_PALETTE_BGR24:
		case WCAM_PALETTE_RGB32:
		case WCAM_PALETTE_BGR32:
			depth = (depth == 24) ? 3 : 4;
//			for (p=img; p < img + width * height * depth; p++) *p=lut[*p];
			for (p=img; p < img + width * height * depth; p+=depth)
			{
				p[0]=pLutR[p[0]];
				p[1]=pLutV[p[1]];
				p[2]=pLutB[p[2]];

				//	Saturation
				if (sat>=0)
				{
					R=p[2]&255; V=p[1]&255; B=p[0]&255;
					Y=(B+3*R+6*V)/10;
					R=Y+(R-Y)*v/4; if (R>255) p[2]=255; else if (R<0) p[2]=0; else p[2]=R;
					R=Y+(V-Y)*v/4; if (R>255) p[1]=255; else if (R<0) p[1]=0; else p[1]=R;
					R=Y+(B-Y)*v/4; if (R>255) p[0]=255; else if (R<0) p[0]=0; else p[0]=R;
				}
				if (neg) { p[0]=255-p[0]; p[1]=255-p[1]; p[2]=255-p[2]; }
				if (inv) { tmp=p[0]; p[0]=p[2]; p[2]=tmp; }
			}
			break;

		case WCAM_PALETTE_UYVY:
			depth = 2;
			for (p=img+1; p < img + width * height * depth; p+=depth) *p=lut[*p];
			break;

		case WCAM_PALETTE_YUYV:
			depth = 2;
			for (p=img+0; p < img + width * height * depth; p+=depth) *p=lut[*p];
			break;
	}
}









void gl860_b2gris8(uint8_t *bayer, uint8_t *bgr,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
//STK_INFO("Farbmodus: 2bgr24.\n");

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;
        int i,j;

	for (j=0; j<nheight-1; j++) {
		for (i=0; i<nwidth-1; i++) {
			*bgr++ = *bayer++;
		}
	}

}

void b2bgr24lattice(unsigned char *bayer, unsigned char *RGB, int IMG_WIDTH, int IMG_HEIGHT) {
	//Simon "the Sorcerers" good old unbayerAlgorithm
 int x,z,i,j;
// STK_INFO("%ix%i.\n",IMG_HEIGHT,IMG_WIDTH);

 for (x=0,z=0,j=0; j < IMG_HEIGHT-2; j+=2){
  for (i=0; i < IMG_WIDTH-2; x+=2,z+=6,i+=2){
   RGB[z+2] = bayer[x+1]; //Blau
   RGB[z+1] = ((bayer[x]+bayer[x+1+IMG_WIDTH])>>1); //Grün
   RGB[z] = bayer[x+IMG_WIDTH]; //Rot
 
   RGB[z+5] = bayer[x+1]; //Blau
   RGB[z+4] = ((bayer[x+2]+bayer[x+1+IMG_WIDTH])>>1); //Grün
   RGB[z+3] = bayer[x+2+IMG_WIDTH]; //Rot
  }
  RGB[z+2] = bayer[x+1]; //Blau
  RGB[z+1] = ((bayer[x]+bayer[x+1+IMG_WIDTH])>>1); //Grün
  RGB[z] = bayer[x+IMG_WIDTH]; //Rot
  z+=3; x+=2;
  z+=3;
 
  for (i=0; i < IMG_WIDTH-2; x+=2,z+=6,i+=2){
   RGB[z+2] = bayer[x+1+IMG_WIDTH]; //Blau
   RGB[z+1] = ((bayer[x+1]+bayer[x+IMG_WIDTH])>>1); //Grün
   RGB[z] = bayer[x]; //Rot
 
   RGB[z+5] = bayer[x+1+IMG_WIDTH]; //Blau
   RGB[z+4] = ((bayer[x+1]+bayer[x+2+IMG_WIDTH])>>1); //Grün
   RGB[z+3] = bayer[x+2]; //Rot
  }
  RGB[z+2] = bayer[x+1+IMG_WIDTH]; //Blau
  RGB[z+1] = ((bayer[x+1]+bayer[x+IMG_WIDTH])>>1); //Grün
  RGB[z] = bayer[x]; //Rot
  z+=3; x+=2;
  z+=3;
 }
 for (i=0; i < IMG_WIDTH-2; x+=2,z+=6,i+=2){
  RGB[z+2] = bayer[x+1]; //Blau
  RGB[z+1] = ((bayer[x]+bayer[x+1+IMG_WIDTH])>>1); //Grün
  RGB[z] = bayer[x+IMG_WIDTH]; //Rot
 
  RGB[z+5] = bayer[x+1]; //Blau
  RGB[z+4] = ((bayer[x+2]+bayer[x+1+IMG_WIDTH])>>1); //Grün
  RGB[z+3] = bayer[x+2+IMG_WIDTH]; //Rot
 }
 RGB[z+2] = bayer[x+1]; //Blau
 RGB[z+1] = ((bayer[x]+bayer[x+1+IMG_WIDTH])>>1); //Grün
 RGB[z] = bayer[x+IMG_WIDTH]; //Rot

}


/** 
 * @brief This function permits to convert an image from bayer to RGB24
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval rgb Buffer with the RGB data
 */
void gl860_b2rgb24(uint8_t *bayer, uint8_t *rgb,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in rgb image

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;

	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}


	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	rgb += ((view->y - nheight) / 2) * view->x * 3;

	// To center horizontally the image in the view
	rgb += ((view->x - nwidth) / 2) * 3;

	// Clean the first line
	memset(rgb, 0, nwidth * 3);
	rgb += nwidth * 3;


	// For each rgb line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		rgb += (view->x - nwidth) * 3;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*rgb++ = *b;
					*rgb++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*rgb++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
				}
				else {
					*rgb++ = (*(b-1) + *(b+1)) >> 1;
					*rgb++ = *b;
					*rgb++ = (*(b-width) + *(b+width)) >> 1;
				}

				b += stepx;
			}

			// Skip the last pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
		}
		else {
			// Skip the first pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*rgb++ = (*(b-width) + *(b+width)) >> 1;
					*rgb++ = *b;
					*rgb++ = (*(b-1) + *(b+1)) >> 1;
				}
				else {
					*rgb++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*rgb++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*rgb++ = *b;
				}
	
				b += stepx;
			}

			// Skip the last pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
		}
	}

	// Clean the last line
	memset(rgb, 0, nwidth * 3);
}


/** 
 * @brief This function permits to convert an image from bayer to RGB32
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval rgb Buffer with the RGB data
 */
void gl860_b2rgb32(uint8_t *bayer, uint8_t *rgb,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in rgb image

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;


	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}


	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	rgb += ((view->y - nheight) / 2) * view->x * 4;

	// To center horizontally the image in the view
	rgb += ((view->x - nwidth) / 2) * 4;

	// Clean the first line
	memset(rgb, 0, nwidth * 4);
	rgb += nwidth * 4;


	// For each rgb line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		rgb += (view->x - nwidth) * 4;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*rgb++ = *b;
					*rgb++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*rgb++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*rgb++ = 0;
				}
				else {
					*rgb++ = (*(b-1) + *(b+1)) >> 1;
					*rgb++ = *b;
					*rgb++ = (*(b-width) + *(b+width)) >> 1;
					*rgb++ = 0;
				}

				b += stepx;
			}

			// Skip the last pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
		}
		else {
			// Skip the first pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*rgb++ = (*(b-width) + *(b+width)) >> 1;
					*rgb++ = *b;
					*rgb++ = (*(b-1) + *(b+1)) >> 1;
					*rgb++ = 0;
				}
				else {
					*rgb++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*rgb++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*rgb++ = *b;
					*rgb++ = 0;
				}
	
				b += stepx;
			}

			// Skip the last pixel
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
			*rgb++ = 0;
		}
	}

	// Clean the last line
	memset(rgb, 0, nwidth * 4);
}


/** 
 * @brief This function permits to convert an image from bayer to BGR24
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval bgr Buffer with the BGR data
 */
void gl860_b2bgr24(uint8_t *bayer, uint8_t *bgr,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in bgr image

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;

	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}


	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	bgr += ((view->y - nheight) / 2) * view->x * 3;

	// To center horizontally the image in the view
	bgr += ((view->x - nwidth) / 2) * 3;

	// Clean the first line
	memset(bgr, 0, nwidth * 3);
	bgr += nwidth * 3;


	// For each bgr line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		bgr += (view->x - nwidth) * 3;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*bgr++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*bgr++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*bgr++ = *b;
				}
				else {
					*bgr++ = (*(b-width) + *(b+width)) >> 1;
					*bgr++ = *b;
					*bgr++ = (*(b-1) + *(b+1)) >> 1;
				}

				b += stepx;
			}

			// Skip the last pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
		}
		else {
			// Skip the first pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*bgr++ = (*(b-1) + *(b+1)) >> 1;
					*bgr++ = *b;
					*bgr++ = (*(b-width) + *(b+width)) >> 1;
				}
				else {
					*bgr++ = *b;
					*bgr++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*bgr++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
				}
	
				b += stepx;
			}

			// Skip the last pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
		}
	}

	// Clean the last line
	memset(bgr, 0, nwidth * 3);
}


/** 
 * @brief This function permits to convert an image from bayer to BGR32
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval bgr Buffer with the BGR data
 */
void gl860_b2bgr32(uint8_t *bayer, uint8_t *bgr,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in bgr image

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;


	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}


	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	bgr += ((view->y - nheight) / 2) * view->x * 4;

	// To center horizontally the image in the view
	bgr += ((view->x - nwidth) / 2) * 4;

	// Clean the first line
	memset(bgr, 0, nwidth * 4);
	bgr += nwidth * 4;


	// For each bgr line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		bgr += (view->x - nwidth) * 4;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*bgr++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*bgr++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*bgr++ = *b;
					*bgr++ = 0;
				}
				else {
					*bgr++ = (*(b-width) + *(b+width)) >> 1;
					*bgr++ = *b;
					*bgr++ = (*(b-1) + *(b+1)) >> 1;
					*bgr++ = 0;
				}

				b += stepx;
			}

			// Skip the last pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
		}
		else {
			// Skip the first pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					*bgr++ = (*(b-1) + *(b+1)) >> 1;
					*bgr++ = *b;
					*bgr++ = (*(b-width) + *(b+width)) >> 1;
					*bgr++ = 0;
				}
				else {
					*bgr++ = *b;
					*bgr++ = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					*bgr++ = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					*bgr++ = 0;
				}
	
				b += stepx;
			}

			// Skip the last pixel
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
			*bgr++ = 0;
		}
	}

	// Clean the last line
	memset(bgr, 0, nwidth * 4);
}


/** 
 * @brief This function permits to convert an image from bayer to YUV (UYVY)
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval yuv Buffer with the YUV data
 */
void gl860_b2uyvy(uint8_t *bayer, uint8_t *yuv,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in yuv image

	int pR, pG, pB;
	int pY, pU, pV;

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;


	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}

	// Background color...
	memset(yuv, 16, width * 2);
	for (i=0; i<width*2; i=i+2, *(yuv+i)=128);
	for (i=1; i<height; i++)
		memcpy(yuv+i*width*2, yuv, width*2);

	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	yuv += ((view->y - nheight) / 2) * view->x * 2;

	// To center horizontally the image in the view
	yuv += ((view->x - nwidth) / 2) * 2;

	// Clean the first line
	memset(yuv, 16, nwidth * 2);
	for (i=0; i<nwidth*2; i=i+2, *(yuv+i)=128);
	yuv += nwidth * 2;


	// For each yuv line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		yuv += (view->x - nwidth) * 2;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*yuv++ = 128;
			*yuv++ = 16;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					pB = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					pG = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					pR = *b;
				}
				else {
					pB = (*(b-width) + *(b+width)) >> 1;
					pG = *b;
					pR = (*(b-1) + *(b+1)) >> 1;
				}

				pY = gl860_yuv_interp[pR][0] + gl860_yuv_interp[pG][1] + gl860_yuv_interp[pB][2];
				pU = gl860_yuv_interp[pR][3] + gl860_yuv_interp[pG][4] + gl860_yuv_interp[pB][5];
				pV = gl860_yuv_interp[pR][5] + gl860_yuv_interp[pG][6] + gl860_yuv_interp[pB][7];

				pY = CLIP(pY, 0,255);
				pU = CLIP(pU, -127,127);
				pV = CLIP(pV, -127,127);
	
				if (i % 2){
					*yuv++ = (112 * pU)/127 + 128; // U
					*yuv++ = (219 * pY)/255 + 16;  // Y
				}
				else {
					*yuv++ = (112 * pV)/127 + 128; // V
					*yuv++ = (219 * pY)/255 + 16;  // Y
				}

				b += stepx;
			}

			// Skip the last pixel
			*yuv++ = 128;
			*yuv++ = 16;
		}
		else {
			// Skip the first pixel
			*yuv++ = 128;
			*yuv++ = 16;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					pB = (*(b-1) + *(b+1)) >> 1;
					pG = *b;
					pR = (*(b-width) + *(b+width)) >> 1;
				}
				else {
					pB = *b;
					pG = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					pR = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
				}

				pY = gl860_yuv_interp[pR][0] + gl860_yuv_interp[pG][1] + gl860_yuv_interp[pB][2];
				pU = gl860_yuv_interp[pR][3] + gl860_yuv_interp[pG][4] + gl860_yuv_interp[pB][5];
				pV = gl860_yuv_interp[pR][5] + gl860_yuv_interp[pG][6] + gl860_yuv_interp[pB][7];

				pY = CLIP(pY, 0,255);
				pU = CLIP(pU, -127,127);
				pV = CLIP(pV, -127,127);
	
				if (i % 2){
					*yuv++ = (112 * pU)/127 + 128; // U
					*yuv++ = (219 * pY)/255 + 16;  // Y
				}
				else {
					*yuv++ = (112 * pV)/127 + 128; // V
					*yuv++ = (219 * pY)/255 + 16;  // Y
				}

				b += stepx;
			}

			// Skip the last pixel
			*yuv++ = 128;
			*yuv++ = 16;
		}
	}

	// Clean the last line
	memset(yuv, 16, nwidth * 2);
	for (i=0; i<nwidth*2; i=i+2, *(yuv+i)=128);
}


/** 
 * @brief This function permits to convert an image from bayer to YUV (YUYV)
 *
 * @param bayer Buffer with the bayer data
 * @param image Size of image
 * @param view Size of view
 * @param hflip Horizontal flip
 * @param vflip Vertical flip
 * @param factor Factor of redimensioning
 *
 * @retval yuv Buffer with the YUV data
 */
void gl860_b2yuyv(uint8_t *bayer, uint8_t *yuv,
		struct gl860_coord *image,
		struct gl860_coord *view,
		const int hflip, const int vflip,
		const int factor) {
	uint8_t *b;

	int x, y; // Position in bayer image
	int i, j; // Position in yuv image

	int pR, pG, pB;
	int pY, pU, pV;

	int width = image->x;
	int height = image->y;

	int nwidth = width / factor;
	int nheight = height / factor;

	int offset;
	int startx, stepx;
	int starty, stepy;


	// Calculate the initial position (on Y axis)
	if (vflip) {
		starty = height - 2;
		stepy = -factor;
	}
	else {
		starty = 0;
		stepy = factor;
	}

	// Calculate the initial position (on X axis)
	if (hflip) {
		startx = width - 1;
		stepx = -factor;
		offset = width - 2;
	}
	else {
		startx = 0;
		stepx = factor;
		offset = 1;
	}

	// Background color...
	memset(yuv, 128, width * 2);
	for (i=0; i<width*2; i=i+2, *(yuv+i)=16);
	for (i=1; i<height; i++)
		memcpy(yuv+i*width*2, yuv, width*2);

	// Skip the first line
	bayer += width;

	// To center vertically the image in the view
	yuv += ((view->y - nheight) / 2) * view->x * 2;

	// To center horizontally the image in the view
	yuv += ((view->x - nwidth) / 2) * 2;

	// Clean the first line
	memset(yuv, 128, nwidth * 2);
	for (i=0; i<nwidth*2; i=i+2, *(yuv+i)=16);
	yuv += nwidth * 2;


	// For each yuv line without the borders (first and last line)
	for (j=0, y=starty; j<nheight-2; j++, y=y+stepy) {
		// Go to the start of line
		b = bayer + y * width + offset;

		// Offset to center horizontally the image in the view
		yuv += (view->x - nwidth) * 2;

		if (!(y & 0x1)) {
			// Skip the first pixel
			*yuv++ = 16;
			*yuv++ = 128;

			// GBGBGB : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					pB = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
					pG = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					pR = *b;
				}
				else {
					pB = (*(b-width) + *(b+width)) >> 1;
					pG = *b;
					pR = (*(b-1) + *(b+1)) >> 1;
				}

				pY = gl860_yuv_interp[pR][0] + gl860_yuv_interp[pG][1] + gl860_yuv_interp[pB][2];
				pU = gl860_yuv_interp[pR][3] + gl860_yuv_interp[pG][4] + gl860_yuv_interp[pB][5];
				pV = gl860_yuv_interp[pR][5] + gl860_yuv_interp[pG][6] + gl860_yuv_interp[pB][7];

				pY = CLIP(pY, 0,255);
				pU = CLIP(pU, -127,127);
				pV = CLIP(pV, -127,127);
	
				if (i % 2){
					*yuv++ = (219 * pY)/255 + 16;  // Y
					*yuv++ = (112 * pU)/127 + 128; // U
				}
				else {
					*yuv++ = (219 * pY)/255 + 16;  // Y
					*yuv++ = (112 * pV)/127 + 128; // V
				}

				b += stepx;
			}

			// Skip the last pixel
			*yuv++ = 16;
			*yuv++ = 128;
		}
		else {
			// Skip the first pixel
			*yuv++ = 16;
			*yuv++ = 128;

			// RGRGRG : Line process...
			for (i=0, x=startx; i<nwidth-2; i++, x=x+stepx) {
				if (x & 0x1) {
					pB = (*(b-1) + *(b+1)) >> 1;
					pG = *b;
					pR = (*(b-width) + *(b+width)) >> 1;
				}
				else {
					pB = *b;
					pG = (*(b-width) + *(b-1) + *(b+1) + *(b+width)) >> 2;
					pR = (*(b-width-1) + *(b-width+1) + *(b+width-1) + *(b+width+1)) >> 2;
				}

				pY = gl860_yuv_interp[pR][0] + gl860_yuv_interp[pG][1] + gl860_yuv_interp[pB][2];
				pU = gl860_yuv_interp[pR][3] + gl860_yuv_interp[pG][4] + gl860_yuv_interp[pB][5];
				pV = gl860_yuv_interp[pR][5] + gl860_yuv_interp[pG][6] + gl860_yuv_interp[pB][7];

				pY = CLIP(pY, 0,255);
				pU = CLIP(pU, -127,127);
				pV = CLIP(pV, -127,127);
	
				if (i % 2){
					*yuv++ = (219 * pY)/255 + 16;  // Y
					*yuv++ = (112 * pU)/127 + 128; // U
				}
				else {
					*yuv++ = (219 * pY)/255 + 16;  // Y
					*yuv++ = (112 * pV)/127 + 128; // V
				}

				b += stepx;
			}

			// Skip the last pixel
			*yuv++ = 16;
			*yuv++ = 128;
		}
	}

	// Clean the last line
	memset(yuv, 128, nwidth * 2);
	for (i=0; i<nwidth*2; i=i+2, *(yuv+i)=16);
}


