/**
* @file gl860-dev-0503.c
* @author Olivier LORIN, tiré du pilote Syntek par Nicolas VIVIEN
* @date 2008-05-05
* @version v0.9.0
*
* @brief Driver for GL860 USB video camera
*
* @note Copyright (C) Olivier LORIN
*
* @par Licences
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <media/v4l2-common.h>

#include "gl860.h"
#include "gl860-dev.h"

#define ctrl_in fct_RTx2
#define ctrl_out fct_RTx2

// Code issu des logs de Ice/Soro2005 (A), Fret_saw/Hulkie(B) et Tricid (C)
// avec l'aide de Kytrix/Blazercist
// Capteur : MI2020

static int exlum=-1;
static int exmirh=-1;
static int exmirv=-1;
static int exnet=-1;
static int excntr=-1;
static int excontrej=-1;
static int exfreq=-1;
static int exgam=-1;

static int init=0;

static void commun(struct usb_gl860 *dev);


char nettete[]={ 0x6c, 0x00, 0x08 };

char lum_a[]={ 0x8c, 0xa2, 0x06 };
char lum_b[]={ 0x90, 0x00, 0x00 };
char lum_c[]={ 0x8c, 0xa1, 0x02 };
char lum_d[]={ 0x90, 0x00, 0x0f };
char lum_e[]={ 0x8c, 0xa1, 0x03 };
char lum_f[]={ 0x90, 0x00, 0x05 };

char rien1[]={ 0x90, 0x00, 0x06 };
char rien2[]={ 0x8c, 0xa1, 0x02 };
char rien3[]={ 0x90, 0x00, 0x1f };

char mir_a[]={ 0x8c, 0x27, 0x19 };
char mir_b[]={ 0x90, 0x04, 0x6c };
char mir_c[]={ 0x8c, 0x27, 0x3b };
char mir_d[]={ 0x90, 0x00, 0x24 };
char mir_e[]={ 0x8c, 0xa1, 0x03 };
char mir_f[]={ 0x90, 0x00, 0x06 };

char freq_a[]={ 0x8c, 0xa4, 0x04 };
char freq_b[]={ 0x90, 0x00, 0x80 };

char cjn_a[]={ 0x8c, 0xa7, 0x00 };
char cjn_b[]={ 0x90, 0x00, 0x00 };
char cjn_c[]={ 0x8c, 0xa7, 0x00 };
char cjn_d[]={ 0x90, 0x00, 0x00 };
char cjn_e[]={ 0x8c, 0xa1, 0x03 };
char cjn_f[]={ 0x90, 0x00, 0x05 };

char t900000[]="\x90\x00\x00";
char t900400[]="\x90\x04\x00";



// Partie commune à l'initialisation et à la mise en marche
static void commun(struct usb_gl860 *dev)
{
if (_0503A_) {
	ctrl_out(dev, 0x40,  1, 0x0000, 0x0000,  0, NULL, "ligne 102"); msleep(1);// msleep(35);
	ctrl_out(dev, 0x40,  1, 0x006a, 0x0007,  0, NULL, "ligne 103");// msleep(35);
	ctrl_out(dev, 0x40,  1, 0x0063, 0x0006,  0, NULL, "ligne 104");// msleep(35);
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL, "ligne 105");// msleep(35);
	ctrl_out(dev, 0x40,  1, 0x0000, 0x00c0,  0, NULL, "ligne 106");
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL, "ligne 107");
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL, "ligne 108");
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL, "ligne 109");
	ctrl_out(dev, 0x40,  1, 0x0004, 0x00d8,  0, NULL, "ligne 110");
	ctrl_out(dev, 0x40,  1, 0x0000, 0x0058,  0, NULL, "ligne 111");
	ctrl_out(dev, 0x40,  1, 0x0002, 0x0004,  0, NULL, "ligne 112");
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL, "ligne 113");
} else {
	if (_0503C_) {
	ctrl_out(dev, 0x40,  1, 0x0008, 0x0004,  0, NULL, "ligne 116"); msleep(35);
	} else {
	ctrl_out(dev, 0x40,  1, 0x0002, 0x0004,  0, NULL, "ligne 118"); msleep(35);
	}
	ctrl_out(dev, 0x40,  1, 0x006a, 0x0007,  0, NULL, "ligne 120"); msleep(35);
	ctrl_out(dev, 0x40,  1, 0x00ef, 0x0006,  0, NULL, "ligne 121"); msleep(35);
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL, "ligne 122"); msleep(35);
	ctrl_out(dev, 0x40,  1, 0x0000, 0x00c0,  0, NULL, "ligne 123");
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL, "ligne 124");
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL, "ligne 125");
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL, "ligne 126");
	ctrl_out(dev, 0x40,  1, 0x0004, 0x00d8,  0, NULL, "ligne 127");
	ctrl_out(dev, 0x40,  1, 0x0000, 0x0058,  0, NULL, "ligne 128");
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL, "ligne 129");
}
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x86\x25\x01", "ligne 131");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x86\x25\x00", "ligne 132"); msleep(2); // - *
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0030,  3, "\x1a\x0a\xcc", "ligne 133"); if (dev->resolution==WCAM_1600x1200) msleep(2); //1600
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x02\x00\x08", "ligne 134");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\xf4\x03\x1d", "ligne 135"); msleep(6); //12 *
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 136");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28", "ligne 137");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 138"); msleep(2); // - *
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 139");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6", "ligne 140");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x23", "ligne 141");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 142");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0f", "ligne 143");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0d", "ligne 144");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x10", "ligne 145");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0b", "ligne 146");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x11", "ligne 147");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x07", "ligne 148");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\xf4\x03\x1d", "ligne 149");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xa2\x00\xe2", "ligne 150");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xab\x05", "ligne 151");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 152");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x6e\x00\x86", "ligne 153");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x70\x0f\xaa", "ligne 154");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x72\x0f\xe4", "ligne 155");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x4a", "ligne 156");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x5a", "ligne 157");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x4b", "ligne 158");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa6", "ligne 159");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x61", "ligne 160");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc8", "ligne 161");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa3\x62", "ligne 162");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xe1", "ligne 163");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8", "ligne 164");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33", "ligne 165");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a", "ligne 166");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63", "ligne 167");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25", "ligne 168");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70", "ligne 169");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c", "ligne 170");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04", "ligne 171");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b", "ligne 172");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13", "ligne 173");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0", "ligne 174");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4", "ligne 175");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7", "ligne 176");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd", "ligne 177");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63", "ligne 178");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c", "ligne 179");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e", "ligne 180");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44", "ligne 181");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4", "ligne 182");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8", "ligne 183");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90", "ligne 184");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8", "ligne 185");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88", "ligne 186");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff", "ligne 187");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52", "ligne 188");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc", "ligne 189");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83", "ligne 190");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3", "ligne 191");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba", "ligne 192");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0", "ligne 193");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63", "ligne 194");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a", "ligne 195");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e", "ligne 196");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28", "ligne 197");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02", "ligne 198");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa", "ligne 199");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef", "ligne 200");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88", "ligne 201");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1", "ligne 202");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64", "ligne 203");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15", "ligne 204");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82", "ligne 205");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce", "ligne 206");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba", "ligne 207");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02", "ligne 208");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3", "ligne 209");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50", "ligne 210");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39", "ligne 211");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c", "ligne 212");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b", "ligne 213");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19", "ligne 214");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08", "ligne 215");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22", "ligne 216");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3", "ligne 217");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54", "ligne 218");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d", "ligne 219");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04", "ligne 220");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0", "ligne 221");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc", "ligne 222");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b", "ligne 223");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2", "ligne 224");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08", "ligne 225");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43", "ligne 226");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62", "ligne 227");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60", "ligne 228");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33", "ligne 229");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0", "ligne 230");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd", "ligne 231");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06", "ligne 232");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff", "ligne 233");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89", "ligne 234");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff", "ligne 235");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0", "ligne 236");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00", "ligne 237");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc", "ligne 238");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18", "ligne 239");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c", "ligne 240"); msleep(1);
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x78\x00\x00", "ligne 241"); msleep(1);
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xb8\x1f\x20", "ligne 242");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x06", "ligne 243");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x10", "ligne 244");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x07", "ligne 245");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x08", "ligne 246");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x42", "ligne 247");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0b", "ligne 248");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x4a", "ligne 249");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x8c", "ligne 250");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\xba\xfa\x08", "ligne 251");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x02", "ligne 252");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x22", "ligne 253");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x03", "ligne 254");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xbb", "ligne 255");
if (_0503A_ || _0503C_) {
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e", "ligne 257");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0", "ligne 258");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08", "ligne 259");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 260");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09", "ligne 261");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x21", "ligne 262");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a", "ligne 263");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25", "ligne 264");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b", "ligne 265");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x27", "ligne 266");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11", "ligne 267");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0", "ligne 268");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13", "ligne 269");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0", "ligne 270");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x15", "ligne 271");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa0", "ligne 272");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17", "ligne 273");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc0", "ligne 274");
}
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04", "ligne 276");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x80", "ligne 277");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d", "ligne 278");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 279");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e", "ligne 280");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 281");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c", "ligne 282");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17", "ligne 283");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x15", "ligne 284");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04", "ligne 285");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x14", "ligne 286");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x20", "ligne 287");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 288");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 289");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x17", "ligne 290");// msleep(53); *
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11", "ligne 291");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1b", "ligne 292");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x4f", "ligne 293");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x25", "ligne 294");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x0f", "ligne 295");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x39", "ligne 296");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11", "ligne 297");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3d", "ligne 298");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x20", "ligne 299");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x47", "ligne 300");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x09\x4c", "ligne 301");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x03", "ligne 302");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x84", "ligne 303");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x05", "ligne 304");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\xe2", "ligne 305");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07", "ligne 306");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40", "ligne 307");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09", "ligne 308");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0", "ligne 309");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x0d", "ligne 310");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 311");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x0f", "ligne 312");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 313");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x11", "ligne 314");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xbd", "ligne 315");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x13", "ligne 316");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x4d", "ligne 317");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x15", "ligne 318");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 319");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x17", "ligne 320");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11", "ligne 321");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19", "ligne 322");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x6c", "ligne 323");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1b", "ligne 324");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x4f", "ligne 325");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1d", "ligne 326");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x02", "ligne 327");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x1f", "ligne 328");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x79", "ligne 329");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x21", "ligne 330");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x55", "ligne 331");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x23", "ligne 332");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x85", "ligne 333");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x25", "ligne 334");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x0f", "ligne 335");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x27", "ligne 336");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x20", "ligne 337");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x29", "ligne 338");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x20", "ligne 339");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2b", "ligne 340");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x10\x20", "ligne 341");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2d", "ligne 342");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x20\x07", "ligne 343");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x2f", "ligne 344");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04", "ligne 345");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x31", "ligne 346");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x04", "ligne 347");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x33", "ligne 348");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xbb", "ligne 349");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x35", "ligne 350");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x4b", "ligne 351");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x37", "ligne 352");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 353");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x39", "ligne 354");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x21\x11", "ligne 355");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b", "ligne 356");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x24", "ligne 357");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3d", "ligne 358");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x20", "ligne 359");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x41", "ligne 360");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x69", "ligne 361");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x45", "ligne 362");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xed", "ligne 363");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x47", "ligne 364");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x09\x4c", "ligne 365");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x51", "ligne 366");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 367");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x53", "ligne 368");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x03\x20", "ligne 369");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x55", "ligne 370");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 371");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x57", "ligne 372");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x58", "ligne 373");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x5f", "ligne 374");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 375");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x61", "ligne 376");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40", "ligne 377");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x63", "ligne 378");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 379");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x65", "ligne 380");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0", "ligne 381");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e", "ligne 382");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa1", "ligne 383");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08", "ligne 384");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 385");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09", "ligne 386");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x21", "ligne 387");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a", "ligne 388");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25", "ligne 389");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b", "ligne 390");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x27", "ligne 391");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11", "ligne 392");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xa1", "ligne 393");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13", "ligne 394");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\xc1", "ligne 395");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x15", "ligne 396");
if (_0503A_ || _0503C_) {
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x78", "ligne 398"); // diff fret
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17", "ligne 399");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x90", "ligne 400"); // diff fret
} else {
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x6a", "ligne 402");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x17", "ligne 403");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x80", "ligne 404");
}
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 406");
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 407"); msleep(2);
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 408"); if (dev->resolution==WCAM_1600x1200) msleep(14); //1600
	ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 409"); msleep(2);
}







//=============================================================================
//
// STK-M811 API / STK-A311 API
//
//=============================================================================

/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to initialize the device.
*
* This function must be called at first. It's the start of the
* initialization process. After this process, the device is
* completly initalized and it's ready.
*
* This function is written from the USB log.
*/
int dev_0503his_initialize_device(struct usb_gl860 *dev)
{
	unsigned char sonde;

	STK_DEBUG("dev_0503his_initialize_device [ini]\n");

	ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sonde, "ligne 443");
	ctrl_in(dev, 0xc0, 2, 0x0000, 0x0004, 1, &sonde, "ligne 444");

	ctrl_out(dev, 0x40, 1, 0x0000, 0x0000, 0, NULL, "ligne 446"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL, "ligne 447"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0008, 0x00c0, 0, NULL, "ligne 448"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL, "ligne 449"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0001, 0x00c2, 0, NULL, "ligne 450"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x0020, 0x0006, 0, NULL, "ligne 451"); msleep(53);
	ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL, "ligne 452"); msleep(53);

	ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL, "ligne 454"); msleep(5);
	ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL, "ligne 455"); msleep(5);
	ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL, "ligne 456");
	ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, &sonde, "ligne 457");
//	STK_INFO("probe 1=%2x\n",sonde);
//
//	ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sonde, "ligne 460");
//	STK_INFO("probe 2=%2x\n",sonde);
//	ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &sonde, "ligne 462");
//	STK_INFO("probe 2=%2x\n",sonde);

	commun(dev);

	STK_DEBUG("dev_0503his_initialize_device [fini]\n");
	return 0;
}


/**
* @param dev Device structure
* @param step The step of configuration [0-11]
*
* @returns 0 if all is OK
*
* @brief This function permits to configure the device.
*
* The configuration of device is composed of 11 steps.
* This function is called by the initialization process.
*
* We don't know the meaning of these steps ! We only replay the USB log.
*
* The steps 0 to 9 are called during the initialization.
* Then, the driver choose the last step :
*   10 : for a resolution from 80x60 to 640x480
*   11 : for a resolution from 800x600 to 1280x1024
*/
int dev_0503his_configure_device(struct usb_gl860 *dev, int step)
{
return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief Wake-up the camera.
 *
 * This function permits to wake-up the device.
 */
int dev_0503his_camera_asleep(struct usb_gl860 *dev)
{
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function initializes the device for the stream.
 *
 * It's the start. This function has to be called at first, before
 * enabling the video stream.
 */
int dev_0503his_init_camera(struct usb_gl860 *dev)
{
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function permits to set default sensor settings.
 *
 * We set some registers in using a I2C bus.
 * WARNING, the sensor settings can be different following the situation.
 */
int dev_0503his_sensor_settings(struct usb_gl860 *dev)
{
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 */
int dev_0503his_camera_settings(struct usb_gl860 *dev)
{
int lum  = (dev->vsettings.brightness/256*129)/256;
int mirh = (dev->vsettings.miroirh > 0);
int mirv = (dev->vsettings.miroirv > 0);
int net	= (dev->vsettings.net/256*41)/256;
int cntr	= (dev->vsettings.contrast/256*4)/256;
int contrej	= (dev->vsettings.contrej/256*65)/256;
int freq	= dev->vsettings.freq50;
int gam	= (dev->vsettings.whiteness/256*3)/256;

	if (dev->nbIm<3) { dev->waitSet=1; return 0; }
	dev->waitSet=0;

	dev->doNotDisturb=1;

	if (freq!=exfreq)
	{
		// Fréquence du courant
		freq_b[2]=(freq)?0xc0:0x80;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)freq_a, "ligne 575");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)freq_b, "ligne 576");
		msleep(20);
		exfreq=freq;
	}

	if (mirh!=exmirh || mirv!=exmirv)
	{
		// Miroirs
		mir_b[2]=0x6c+2*(1-mirv)+(1-mirh);
		mir_d[2]=0x24+2*(1-mirv)+(1-mirh);

		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_a, "ligne 587"); // 6 *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_b, "ligne 588"); // 6 *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_c, "ligne 589"); // 6 *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_d, "ligne 590"); // 6 *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_e, "ligne 591"); // 6 *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)mir_f, "ligne 592"); // 6 *
		msleep(130);
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 594"); msleep(6); // - *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 595"); msleep(6); // - *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 596"); msleep(6); // - *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 597"); msleep(6); // - *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 598"); msleep(6); // - *
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien1, "ligne 599"); msleep(6); // - *
/*
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2, "ligne 601");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3, "ligne 602");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2, "ligne 603");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3, "ligne 604");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2, "ligne 605");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3, "ligne 606");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien2, "ligne 607");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)rien3, "ligne 608");
*/
		exmirh=mirh;
		exmirv=mirv;
	}

	if (contrej!=excontrej)
	{
		// Contre-jour
		cjn_a[2]=0x9d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=contrej;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a, "ligne 619");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b, "ligne 620");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c, "ligne 621");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d, "ligne 622");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e, "ligne 623");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f, "ligne 624");
		excontrej=contrej;
	}

	if (gam!=exgam)
	{
		// Gamma
		cjn_a[2]=0x6d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=0x40+gam;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a, "ligne 646");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b, "ligne 647");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c, "ligne 648");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d, "ligne 649");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e, "ligne 650");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f, "ligne 651");
		exgam=gam;
	}

	if (cntr!=excntr)
	{
		// Contraste
		cjn_a[2]=0x6d; cjn_c[2]=cjn_a[2]+1;
		cjn_d[2]=cjn_b[2]=0x12+16*cntr;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_a, "ligne 632");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_b, "ligne 633");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_c, "ligne 634");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_d, "ligne 635");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_e, "ligne 636");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)cjn_f, "ligne 637");
		excntr=cntr;
	}

	if (lum!=exlum)
	{
		// Luminosité
		lum_b[2]=lum;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_a, "ligne 659");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_b, "ligne 660");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_c, "ligne 661");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_d, "ligne 662");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_e, "ligne 663");
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0033, 3, (void*)lum_f, "ligne 664");
		exlum=lum;
	}

	if (net!=exnet)
	{
		// Netteté
		nettete[1]=(unsigned char)net;
		ctrl_out(dev, 0x40, 3, 0x7a00, 0x0032, 3, (void*)nettete, "ligne 672");
		exnet=net;
	}

	dev->doNotDisturb=0;

	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the settings :
 *   - brightness
 *   - contrast
 *   - white balance
 *   - ...
 *
 */
int dev_0503his_set_camera_quality(struct usb_gl860 *dev)
{
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function permits to modify the settings of the camera.
 *
 * This functions permits to modify the frame rate per second.
 *
 */
int dev_0503his_set_camera_fps(struct usb_gl860 *dev)
{
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to start the stream.
 *
 * After the initialization of the device and the initialization of the video stream,
 * this function permits to enable the stream.
 */
int dev_0503his_start_stream(struct usb_gl860 *dev)
{
int mirh = (dev->vsettings.miroirh > 0);
int mirv = (dev->vsettings.miroirv > 0);
int contrej	= (dev->vsettings.contrej/256*65)/256;
int freq	= dev->vsettings.freq50;
char c;
	STK_DEBUG("dev_0503his_start_stream [ini]\n");

	dev->nbIm=-1;

	exlum=-1;
	exmirh=-1;
	exmirv=-1;
	exnet=-1;
	excntr=-1;
	excontrej=-1;
	exfreq=-1;
	exgam=-1;
	dev->initFinie = 0;
	init=1;

	msleep(200);

	ctrl_out(dev, 0x40, 5, 0x0001, 0x0000, 0, NULL, "ligne 746"); msleep(3); // 35 *

	commun(dev);

if (_0503A_) {
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL, "ligne 751"); msleep(70);
	ctrl_out(dev, 0x40,  1, 0x0040, 0x0000,  0, NULL, "ligne 752");
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL, "ligne 753");
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL, "ligne 754");
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL, "ligne 755");
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL, "ligne 756");
} else {
	ctrl_out(dev, 0x40,  1, 0x0041, 0x0000,  0, NULL, "ligne 758"); msleep(70);
	ctrl_out(dev, 0x40,  1, 0x0010, 0x0010,  0, NULL, "ligne 759");
	ctrl_out(dev, 0x40,  1, 0x0003, 0x00c1,  0, NULL, "ligne 760");
	ctrl_out(dev, 0x40,  1, 0x0042, 0x00c2,  0, NULL, "ligne 761");
	ctrl_out(dev, 0x40,  1, 0x006a, 0x000d,  0, NULL, "ligne 762");
}

	switch (dev->resolution) {
		case WCAM_800x600:
		case WCAM_640x480:
		case WCAM_320x240:
		case WCAM_160x120:
		case WCAM_213x160:
		case WCAM_128x96:
		case WCAM_80x60:
			if (dev->resolution!=WCAM_800x600)
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x08\xd2\xe1\xd3\x02\xd4\x10\xd5\x81", "ligne 774");
			else
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x10\xd2\x57\xd3\x02\xd4\x18\xd5\x21", "ligne 776");

		if (_0503B_) {
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x15", "ligne 779");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x25", "ligne 780");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x22\x2e", "ligne 781");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x81", "ligne 782");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x08", "ligne 783");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17", "ligne 784");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x09", "ligne 785");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1a", "ligne 786");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0a", "ligne 787");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1d", "ligne 788");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x0b", "ligne 789");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x20", "ligne 790");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x11", "ligne 791");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x81", "ligne 792");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x24\x13", "ligne 793");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x9b", "ligne 794");
		}

			if (dev->resolution==WCAM_800x600)
			{
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x03", "ligne 799");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x03\x24", "ligne 800");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x05", "ligne 801");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x02\x58", "ligne 802");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 803");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 804"); msleep(2);
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 805");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 806"); msleep(2);
			}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 809");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28", "ligne 810");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 811"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 812");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6", "ligne 813");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 814");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 815");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 816");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 817");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x2e\x01\x00", "ligne 818");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x04\x00\x2a", "ligne 819");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 820");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 821");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x95", "ligne 822");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x00", "ligne 823"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 824");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72", "ligne 825");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 826");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02", "ligne 827");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 828");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 829"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 830");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 831");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 832");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 833");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 834");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 835"); msleep(2); // - *
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 836");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 837"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 838");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 839"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 840");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 841"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 842");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 843"); msleep(1);

		if (_0503A_) {
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL, "ligne 846");
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL, "ligne 847");
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL, "ligne 848"); msleep(150);
		} else if (_0503B_) {
			ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL, "ligne 850");
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL, "ligne 851");
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL, "ligne 852"); msleep(120);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL, "ligne 853"); msleep(30);
		} else if (_0503C_) {
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL, "ligne 855");
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL, "ligne 856");
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL, "ligne 857"); msleep(120);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL, "ligne 858"); msleep(30);
		}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04", "ligne 861"); t900000[2]=freq?0xc0:0x80; // freq=50Hz
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 862"); msleep(20); // "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d", "ligne 863"); t900000[2]=contrej; // contrej=0
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 864");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e", "ligne 865");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 866");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c", "ligne 867");// ?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17", "ligne 868");// ?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 869");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 870"); msleep(5);// "

		if (_0503B_) {
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xf8", "ligne 873");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8", "ligne 874");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33", "ligne 875");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a", "ligne 876");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63", "ligne 877");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25", "ligne 878");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70", "ligne 879");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c", "ligne 880");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04", "ligne 881");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b", "ligne 882");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13", "ligne 883");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0", "ligne 884");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4", "ligne 885");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7", "ligne 886");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd", "ligne 887");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63", "ligne 888");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c", "ligne 889");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e", "ligne 890");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44", "ligne 891");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4", "ligne 892");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8", "ligne 893");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90", "ligne 894");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8", "ligne 895");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88", "ligne 896");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff", "ligne 897");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52", "ligne 898");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc", "ligne 899");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83", "ligne 900");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3", "ligne 901");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba", "ligne 902");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0", "ligne 903");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63", "ligne 904");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a", "ligne 905");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e", "ligne 906");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28", "ligne 907");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02", "ligne 908");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa", "ligne 909");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef", "ligne 910");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88", "ligne 911");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1", "ligne 912");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64", "ligne 913");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15", "ligne 914");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82", "ligne 915");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce", "ligne 916");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba", "ligne 917");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02", "ligne 918");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3", "ligne 919");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50", "ligne 920");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39", "ligne 921");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c", "ligne 922");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b", "ligne 923");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19", "ligne 924");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08", "ligne 925");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22", "ligne 926");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3", "ligne 927");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54", "ligne 928");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d", "ligne 929");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04", "ligne 930");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0", "ligne 931");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc", "ligne 932");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b", "ligne 933");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2", "ligne 934");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08", "ligne 935");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43", "ligne 936");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62", "ligne 937");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60", "ligne 938");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33", "ligne 939");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0", "ligne 940");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd", "ligne 941");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06", "ligne 942");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff", "ligne 943");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89", "ligne 944");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff", "ligne 945");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0", "ligne 946");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00", "ligne 947");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc", "ligne 948");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18", "ligne 949");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c", "ligne 950");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19", "ligne 951");// Miroir h/v=1
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x6c", "ligne 952");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b", "ligne 953");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x24", "ligne 954");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 955");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 956"); msleep(130); // "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 957");// "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 958");// "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 959");// "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 960");// "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 961");// "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 962"); msleep(100); // "?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 963");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 964");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 965");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 966");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 967");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 968");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 969");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 970");// "??
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x06", "ligne 971");// Lum=70
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x46", "ligne 972");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 973");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x0f", "ligne 974");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 975");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 976");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x6c\x14\x08", "ligne 977");// Sharpness=20
		} else
		{
			ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &c, "ligne 980"); msleep(14);//0xd8

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19", "ligne 982"); t900400[2]=0x6c+2*(1-mirv)+(1-mirh);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900400, "ligne 983");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b", "ligne 984"); t900000[2]=0x24+2*(1-mirv)+(1-mirh);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 985");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 986");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 987"); msleep(21); 
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 988"); msleep(5); // 6 *
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 989"); msleep(5);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 990"); msleep(5);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 991"); msleep(5);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 992"); msleep(5);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 993"); msleep(246); // 146 *
			dev->nbIm=0;
		}
			break;

		case WCAM_1280x960:
		case WCAM_1600x1200:
			if (dev->resolution==WCAM_1280x960)
			{
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x20\xd2\x01\xd3\x02\xd4\x28\xd5\x01", "ligne 1001");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07", "ligne 1002");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x05\x04", "ligne 1003");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09", "ligne 1004");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\x02", "ligne 1005");
			}
			else
			{
				ctrl_out(dev, 0x40,  3, 0x0000, 0x0200, 12, "\xd0\x02\xd1\x20\xd2\xaf\xd3\x02\xd4\x30\xd5\x41", "ligne 1009");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x07", "ligne 1010");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x06\x40", "ligne 1011");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x09", "ligne 1012");
				ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x04\xb0", "ligne 1013");
			}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1016");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 1017"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1018");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 1019"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 1020");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1c\x01\x28", "ligne 1021");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 1022");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x1e\x8f\x09", "ligne 1023");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x14\x06\xe6", "ligne 1024");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1025");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 1026"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1027");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 1028"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1029");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 1030"); msleep(2);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1031");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 1032");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 1033");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72", "ligne 1034");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x30", "ligne 1035");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03", "ligne 1036");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x31", "ligne 1037");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02", "ligne 1038");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x32", "ligne 1039");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03", "ligne 1040");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x34", "ligne 1041");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x03", "ligne 1042");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1043");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02", "ligne 1044");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x2e\x01\x00", "ligne 1045");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\x04\x00\x2a", "ligne 1046");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 1047");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 1048"); if (dev->resolution==WCAM_1600x1200) msleep(13); //1600
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x97", "ligne 1049");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x01\x00", "ligne 1050"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 1051");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 1052");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1053");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 1054"); if (dev->resolution==WCAM_1600x1200) msleep(13); //1600
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 1055");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 1056"); msleep(53);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 1057");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72", "ligne 1058");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1059");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02", "ligne 1060"); if (dev->resolution==WCAM_1600x1200) msleep(13); //1600
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 1061");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 1062"); msleep(53);

		if (_0503A_) {
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL, "ligne 1065"); if (dev->resolution==WCAM_1600x1200) msleep(500); //1600
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL, "ligne 1066");
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL, "ligne 1067"); msleep(1850);
		} else if (_0503B_ || _0503C_) {
			ctrl_out(dev, 0x40, 1, 0x0001, 0x0010, 0, NULL, "ligne 1069");
			ctrl_out(dev, 0x40, 1, 0x0000, 0x00c1, 0, NULL, "ligne 1070");
			ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL, "ligne 1071"); msleep(1850);
			ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL, "ligne 1072"); msleep(30);
		}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa4\x04", "ligne 1075"); t900000[2]=freq?0xc0:0x80; // freq 50Hz
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 1076"); msleep(20); // "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9d", "ligne 1077"); t900000[2]=contrej; // contrej=0
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 1078");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x9e", "ligne 1079");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 1080");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa2\x0c", "ligne 1081");// ?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x17", "ligne 1082");// ?
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1083");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x05", "ligne 1084"); msleep(6);// "

			ctrl_in(dev, 0xc0,  2, 0x0000, 0x0000,  1, &c, "ligne 1086"); msleep(14);

		if (_0503B_) {
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xf8", "ligne 1089");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xce\x01\xa8", "ligne 1090");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd0\x66\x33", "ligne 1091");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd2\x31\x9a", "ligne 1092");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd4\x94\x63", "ligne 1093");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd6\x4b\x25", "ligne 1094");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xd8\x26\x70", "ligne 1095");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xda\x72\x4c", "ligne 1096");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xdc\xff\x04", "ligne 1097");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xde\x01\x5b", "ligne 1098");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe6\x01\x13", "ligne 1099");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xee\x0b\xf0", "ligne 1100");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf6\x0b\xa4", "ligne 1101");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x00\xf6\xe7", "ligne 1102");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x08\x0d\xfd", "ligne 1103");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x10\x25\x63", "ligne 1104");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x18\x35\x6c", "ligne 1105");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x20\x42\x7e", "ligne 1106");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x28\x19\x44", "ligne 1107");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x30\x39\xd4", "ligne 1108");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x38\xf5\xa8", "ligne 1109");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4c\x07\x90", "ligne 1110");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x44\x07\xb8", "ligne 1111");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5c\x06\x88", "ligne 1112");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x54\x07\xff", "ligne 1113");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe0\x01\x52", "ligne 1114");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe8\x00\xcc", "ligne 1115");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf0\x0d\x83", "ligne 1116");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf8\x0c\xb3", "ligne 1117");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x02\xfe\xba", "ligne 1118");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0a\x04\xe0", "ligne 1119");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x12\x1c\x63", "ligne 1120");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1a\x2b\x5a", "ligne 1121");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x22\x32\x5e", "ligne 1122");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2a\x0d\x28", "ligne 1123");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x32\x2c\x02", "ligne 1124");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3a\xf4\xfa", "ligne 1125");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4e\x07\xef", "ligne 1126");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x46\x07\x88", "ligne 1127");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5e\x07\xc1", "ligne 1128");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x56\x04\x64", "ligne 1129");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe4\x01\x15", "ligne 1130");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xec\x00\x82", "ligne 1131");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf4\x0c\xce", "ligne 1132");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfc\x0c\xba", "ligne 1133");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x06\x1f\x02", "ligne 1134");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0e\x02\xe3", "ligne 1135");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x16\x1a\x50", "ligne 1136");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1e\x24\x39", "ligne 1137");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x26\x23\x4c", "ligne 1138");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2e\xf9\x1b", "ligne 1139");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x36\x23\x19", "ligne 1140");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3e\x12\x08", "ligne 1141");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x52\x07\x22", "ligne 1142");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x4a\x03\xd3", "ligne 1143");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x62\x06\x54", "ligne 1144");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x5a\x04\x5d", "ligne 1145");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xe2\x01\x04", "ligne 1146");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xea\x00\xa0", "ligne 1147");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xf2\x0c\xbc", "ligne 1148");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0034,  3, "\xfa\x0c\x5b", "ligne 1149");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x04\x17\xf2", "ligne 1150");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x0c\x02\x08", "ligne 1151");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x14\x28\x43", "ligne 1152");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x1c\x28\x62", "ligne 1153");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x24\x2b\x60", "ligne 1154");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x2c\x07\x33", "ligne 1155");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x34\x1f\xb0", "ligne 1156");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x3c\xed\xcd", "ligne 1157");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x50\x00\x06", "ligne 1158");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x48\x07\xff", "ligne 1159");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x60\x05\x89", "ligne 1160");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x58\x07\xff", "ligne 1161");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x40\x00\xa0", "ligne 1162");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0035,  3, "\x42\x00\x00", "ligne 1163");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0032,  3, "\x10\x01\xfc", "ligne 1164");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x18", "ligne 1165");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x3c", "ligne 1166");
		}

			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x19", "ligne 1169"); t900400[2]=0x6c+2*(1-mirv)+(1-mirh);// miroir h/v=0
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900400, "ligne 1170");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\x27\x3b", "ligne 1171"); t900000[2]=0x24+2*(1-mirv)+(1-mirh);// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, t900000, "ligne 1172");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1173");// "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x06", "ligne 1174"); msleep(16); // "
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1175");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 1176");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 1177");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 1178"); if (dev->resolution==WCAM_1600x1200) msleep(25); //1600
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 1179");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x00", "ligne 1180"); msleep(103);
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x03", "ligne 1181");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x02", "ligne 1182");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x20", "ligne 1183");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x72", "ligne 1184");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa7\x02", "ligne 1185");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x01", "ligne 1186");
			dev->nbIm=0;

		if (_0503B_) {
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 1189");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 1190");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 1191");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 1192");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 1193");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 1194");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x8c\xa1\x02", "ligne 1195");
			ctrl_out(dev, 0x40,  3, 0x7a00, 0x0033,  3, "\x90\x00\x1f", "ligne 1196");
		}
	}

	exmirh=mirh;
	exmirv=mirv;
	exfreq=freq;
	excontrej=contrej;

	dev->initFinie = 1;

	STK_DEBUG("dev_0503his_start_stream [fini]\n");
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief Reconfigure the camera before the stream.
 *
 * Before enabling the video stream, you have to reconfigure the device.
 */
int dev_0503his_reconf_camera(struct usb_gl860 *dev)
{
	switch (dev->resolution) {
		case WCAM_640x480:
		case WCAM_320x240:
		case WCAM_160x120:
		case WCAM_213x160:
		case WCAM_128x96:
		case WCAM_80x60:
			dev->lealt=3;
			dev->offset=2048;
			dev->nbPaq=32;
			break;

		case WCAM_800x600:
			dev->lealt=1;
			dev->offset=3072;
			dev->nbPaq=32;
			break;

		case WCAM_1280x960:
			dev->lealt=1;
			dev->offset=3072;
			dev->nbPaq=32;
			break;

		case WCAM_1600x1200:
			dev->lealt=1;
			dev->offset=3072;
			dev->nbPaq=32;
			break;
	}
	return 0;
}


/**
 * @param dev Device structure
 *
 * @returns 0 if all is OK
 *
 * @brief This function sets the device to stop the stream.
 *
 * You use the function start_stream to enable the video stream. So you
 * have to use the function stop_strem to disable the video stream.
 */
int dev_0503his_stop_stream(struct usb_gl860 *dev)
{
	STK_DEBUG("dev_0503his_stop_stream [ini]\n");

if (_0503B_ || _0503C_) {
	ctrl_out(dev, 0x40, 5, 0x0000, 0x0000, 0, NULL, "ligne 1265"); msleep(20);
	ctrl_out(dev, 0x40, 1, 0x0001, 0x0000, 0, NULL, "ligne 1266");
} else {
	ctrl_out(dev, 0x40, 5, 0x0000, 0x0000, 0, NULL, "ligne 1268"); msleep(20);
	ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL, "ligne 1269");
}

	STK_DEBUG("dev_0503his_stop_stream [fini]\n");
	return 0;
}

