/**
* @file gl860-dev-0503.c
* @author Olivier LORIN, tiré des logs de Malmostoso et du pilote Syntek par Nicolas VIVIEN
* @date 2008-05-05
* @version v0.9.0
*
* @brief Driver for GL860 USB video camera
*
* @note Copyright (C) Olivier LORIN
*
* @par Licences
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
*/

#include <linux/module.h>
#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/kref.h>

#include <linux/usb.h>
#include <media/v4l2-common.h>

#include "gl860.h"
#include "gl860-dev.h"

#define ctrl_in fct_RTx
#define ctrl_out fct_RTx

// Code issu de logs de Malmostoso
// Capteur : OV2640

static int excj   = -1;
static int exnet  = -1;
static int exsat  = -1;
static int excoul = -1;
static int exsrc  = -1;
static int excntr = -1;
static int exgam  = -1;
static int exlum  = -1;

char msi005[]={ 0xff }; // ATTENDU
char msi006[]={ 0xff }; // ATTENDU
char msi007[]={ 0xff }; // ATTENDU
char msi008[]={ 0xff }; // ATTENDU
char ms26[]={ 0x26 }; // ATTENDU
char ms63[]={ 0x63 }; // ATTENDU

char ms56[]={ 0x56 }; // ATTENDU
char ms5c[]={ 0x5c }; // ATTENDU
char ms75[]={ 0x75 }; // ATTENDU
char ms76[]={ 0x76 }; // ATTENDU
char ms77[]={ 0x77 }; // ATTENDU
char msd2[]={ 0xd2 }; // ATTENDU
char msde[]={ 0xde }; // ATTENDU

char msi321[]={ 0x00, 0x41, 0x07, 0x6a, 0x06, 0x61, 0x0d, 0x6a, 0x10, 0x10, 0xc1, 0x01 };
char ms61[]={ 0x61 }; // ATTENDU
char ms51[]={ 0x51 }; // ATTENDU

/*
char msc000[]={ 0x00, 0x41, 0x07, 0x6a, 0x06, 0x61, 0x0d, 0x6a, 0x10, 0x10, 0xc1, 0x01 };
char msc001[]={ 0x61 }; // ATTENDU
char msc002[]={ 0x51 }; // ATTENDU
char msc003[]={ 0x51 }; // ATTENDU
char msc004[]={ 0xd0, 0x01, 0xd1, 0x08, 0xd2, 0xe0, 0xd3, 0x02, 0xd4, 0x10, 0xd5, 0x81 };
char msc006[]={ 0x28 }; // ATTENDU
char msc007[]={ 0xa8 }; // ATTENDU
*/
//char msc008[]={ 0x50 }; // ATTENDU

char mscommun[]={ 0x00, 0x41, 0x07, 0x6a, 0x06, 0xef, 0x0d, 0x6a, 0x10, 0x10, 0xc1, 0x01 };
char ms640[] ={ 0xd0, 0x01, 0xd1, 0x08, 0xd2, 0xe0, 0xd3, 0x02, 0xd4, 0x10, 0xd5, 0x81 };
char ms800[] ={ 0xd0, 0x01, 0xd1, 0x10, 0xd2, 0x58, 0xd3, 0x02, 0xd4, 0x18, 0xd5, 0x21 };
char ms1280[]={ 0xd0, 0x01, 0xd1, 0x18, 0xd2, 0xc0, 0xd3, 0x02, 0xd4, 0x28, 0xd5, 0x01 };
char ms1600[]={ 0xd0, 0x01, 0xd1, 0x20, 0xd2, 0xb0, 0xd3, 0x02, 0xd4, 0x30, 0xd5, 0x41 };

char ms50[]={ 0x50 }; // ATTENDU
char ms28[]={ 0x28 }; // ATTENDU
char msa8[]={ 0xa8 }; // ATTENDU
char msd8[]={ 0xd8 }; // ATTENDU

//=============================================================================
//
// STK-M811 API / STK-A311 API
//
//=============================================================================

static void commun(struct usb_gl860 *dev)
{
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x002c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60df, 0x002e, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x0012, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0045, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6035, 0x003c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6028, 0x0004, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60e5, 0x0013, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0014, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600c, 0x002c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6078, 0x0033, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f7, 0x003b, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x003e, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x0043, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6010, 0x0016, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6082, 0x0039, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0035, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600a, 0x0022, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6040, 0x0037, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0023, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a0, 0x0034, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601a, 0x0036, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c0, 0x0007, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60b7, 0x000d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x000e, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x004c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6081, 0x004a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6099, 0x0021, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0009, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x603e, 0x0024, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6034, 0x0025, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6081, 0x0026, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0045, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x005c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0063, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6070, 0x0061, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x0062, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x0020, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6030, 0x0028, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x006c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x006e, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0070, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6094, 0x0071, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c1, 0x0073, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6034, 0x003d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6057, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60bb, 0x004f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x609c, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x006d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0039, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6033, 0x003a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x003b, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6031, 0x003c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6014, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x0076, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a0, 0x0033, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6020, 0x0042, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6018, 0x0043, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x004c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60d0, 0x0087, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600f, 0x0088, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6003, 0x00d7, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6010, 0x00d9, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x00da, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6082, 0x00d3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c0, 0x00f9, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6006, 0x0044, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6007, 0x00d1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x00d2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00d2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x00d8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6008, 0x00c8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x00c9, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6008, 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6020, 0x007d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6020, 0x007d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0090, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600e, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601a, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6031, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x605a, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6069, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6075, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x607e, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x608f, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6096, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a3, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60af, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c4, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60d7, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60e8, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6020, 0x0091, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0092, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6006, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60e3, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0096, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6008, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6019, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600c, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6024, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6030, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6028, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6026, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6098, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ed, 0x00c3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x609a, 0x00c4, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00a4, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x00c5, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6051, 0x00c6, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6010, 0x00c7, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6066, 0x00b6, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a5, 0x00b8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6064, 0x00b7, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x607c, 0x00b9, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60af, 0x00b3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6097, 0x00b4, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00b5, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c5, 0x00b0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6094, 0x00b1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600f, 0x00b2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x605c, 0x00c4, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00a8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c8, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6096, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601d, 0x0086, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6090, 0x0051, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6018, 0x0052, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0053, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0054, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0055, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0057, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6090, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6018, 0x005b, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x005c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ed, 0x00c3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x007f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x00da, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601f, 0x00e5, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6067, 0x00e1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0005, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0045, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0010, 0, NULL);
}




/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to initialize the device.
*
* This function must be called at first. It's the start of the
* initialization process. After this process, the device is
* completly initalized and it's ready.
*
* This function is written from the USB log.
*/
int dev_0503ms_initialize_device(struct usb_gl860 *dev)
{
STK_DEBUG("dev_0503ms_initialize_device [ini]n");


ctrl_out(dev, 0x40, 1, 0x0000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0008, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0001, 0x00c2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0020, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL);
/*
ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, msff);
STK_INFO("probe=%2xn",(*msi005)&0xFF);

ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, msff);

ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, msff);

ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0063, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x7a00, 0x8030, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x7a00, 0x8030, 1, msff);
*/
ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x800a, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x6000, 0x800a, 1, ms26);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0006, 1, ms63);

ctrl_out(dev, 0x40, 1, 0x00ef, 0x0006, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms50);

ctrl_out(dev, 0x40, 1, 0x00de, 0x0000, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msde);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms77);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms76);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0007, 1, ms75);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x00d2, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, msd2);

ctrl_out(dev, 0x40, 1, 0x0056, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms56);

ctrl_out(dev, 0x40, 1, 0x005c, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms5c);

ctrl_out(dev, 0x40, 1, 0x00d8, 0x0000, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)msd8);

ctrl_out(dev, 0x40, 1, 0x0050, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x006a, 0x0007, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0061, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0000, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0004, 0x00d8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0012, 0x0004, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0000, 0x0058, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0061, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)msi321);

commun(dev);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0006, 1, ms61);

ctrl_out(dev, 0x40, 1, 0x00ef, 0x0006, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, ms51);

ctrl_out(dev, 0x40, 1, 0x0051, 0x0000, 0, NULL);
//	ctrl_out(dev, 0x40, 11, 0x0000, 0x0000, 0, NULL);


STK_DEBUG("dev_0503ms_initialize_device [fini]n");
return 0;
}


/**
* @param dev Device structure
* @param step The step of configuration [0-11]
*
* @returns 0 if all is OK
*
* @brief This function permits to configure the device.
*
* The configuration of device is composed of 11 steps.
* This function is called by the initialization process.
*
* We don't know the meaning of these steps ! We only replay the USB log.
*
* The steps 0 to 9 are called during the initialization.
* Then, the driver choose the last step :
*   10 : for a resolution from 80x60 to 640x480
*   11 : for a resolution from 800x600 to 1280x1024
*/
int dev_0503ms_configure_device(struct usb_gl860 *dev, int step)
{
return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief Wake-up the camera.
*
* This function permits to wake-up the device.
*/
int dev_0503ms_camera_asleep(struct usb_gl860 *dev)
{
return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function initializes the device for the stream.
*
* It's the start. This function has to be called at first, before
* enabling the video stream.
*/
int dev_0503ms_init_camera(struct usb_gl860 *dev)
{
return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to set default sensor settings.
*
* We set some registers in using a I2C bus.
* WARNING, the sensor settings can be different following the situation.
*/
int dev_0503ms_sensor_settings(struct usb_gl860 *dev)
{
excj   = -1;
exnet  = -1;
exsat  = -1;
excoul = -1;
exsrc  = -1;
excntr = -1;
exgam  = -1;
exlum  = -1;

ctrl_out(dev, 0x40, 5, 0x0001, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x006a, 0x0007, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x00ef, 0x0006, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0000, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0010, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0001, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x00c2, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0004, 0x00d8, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0012, 0x0004, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0000, 0x0058, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x0041, 0x0000, 0, NULL); msleep(50);
ctrl_out(dev, 0x40, 1, 0x0061, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)mscommun);

commun(dev);

ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6007, 0x007d, 0, NULL);
msleep(30);
ctrl_out(dev, 0x40, 1, 0x0040, 0x0000, 0, NULL);

switch (dev->resolution) {
case WCAM_640x480:
case WCAM_320x240:
case WCAM_160x120:
case WCAM_213x160:
case WCAM_128x96:
case WCAM_80x60:
dev->lealt=3;
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6067, 0x00e1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00da, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0012, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x0017, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6075, 0x0018, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x0019, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6097, 0x001a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6036, 0x0032, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60bb, 0x004f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6057, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x609c, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x006d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6092, 0x0026, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x0020, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0027, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c8, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6096, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x008c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x603d, 0x0086, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6089, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6090, 0x0051, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x602c, 0x0052, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0053, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0054, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0055, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0057, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a0, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6078, 0x005b, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x005c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00d3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a1, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)ms640);
break;

case WCAM_800x600:
dev->lealt=1;
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6067, 0x00e1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00da, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6040, 0x0012, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x0017, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6043, 0x0018, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0019, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x604b, 0x001a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6009, 0x0032, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ca, 0x004f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60a8, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x006d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6038, 0x003d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c8, 0x0035, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0022, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6092, 0x0026, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x0020, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0027, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6064, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x604b, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x008c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601d, 0x0086, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6082, 0x00d3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6020, 0x008c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6044, 0x0018, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)ms800);
break;

case WCAM_1600x1200:
case WCAM_1280x960:
dev->lealt=1;
ctrl_out(dev, 0x40, 1, 0x0002, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0012, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0045, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6011, 0x0017, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6075, 0x0018, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x0019, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6097, 0x001a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6036, 0x0032, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60bb, 0x004f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x609c, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6057, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6080, 0x006d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6043, 0x000f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x608f, 0x0003, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6081, 0x0026, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c8, 0x00c0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6096, 0x00c1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x008c, 0, NULL);

if (dev->resolution==WCAM_1280x960)
{
ctrl_out(dev, 0x40, 1, 0x603d, 0x0086, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0050, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6090, 0x0051, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x602c, 0x0052, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0053, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0054, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6088, 0x0055, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0057, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6040, 0x005a, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f0, 0x005b, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x005c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6082, 0x00d3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x008e, 0, NULL);
} else {
ctrl_out(dev, 0x40, 1, 0x601d, 0x0086, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00d7, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6082, 0x00d3, 0, NULL);
}

ctrl_out(dev, 0x40, 1, 0x6004, 0x00da, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6067, 0x00e1, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60f1, 0x00dd, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6004, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6010, 0x00c7, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0092, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6006, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60e3, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6005, 0x0093, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ed, 0x00c3, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00a4, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60d0, 0x0087, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6003, 0x0096, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600c, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6024, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6030, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6028, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6026, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0097, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6043, 0x000f, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x608f, 0x0003, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x002d, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x002e, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x600a, 0x0022, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002, 0x0070, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6008, 0x0014, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6048, 0x0014, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x00e0, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60ff, 0x00dd, 0, NULL);

if (dev->resolution==WCAM_1280x960)
{
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)ms1280);
} else {
ctrl_out(dev, 0x40, 1, 0x6020, 0x008c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6076, 0x0018, 0, NULL);
ctrl_out(dev, 0x40, 3, 0x0000, 0x0200, 12, (void*)ms1600);
}
break;
}

// COMMUN
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6038, 0x000c, 0, NULL);
msleep(10);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0011, 0, NULL);//en dessous avant
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);//au dessus avant//c
ctrl_out(dev, 0x40, 1, 0x603e, 0x0024, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6034, 0x0025, 0, NULL);//cj=31/64

ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//l
ctrl_out(dev, 0x40, 1, 0x6009, 0x007c, 0, NULL);//l
ctrl_out(dev, 0x40, 1, 0x6000, 0x007d, 0, NULL);//lum=0/256
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//s
ctrl_out(dev, 0x40, 1, 0x6003, 0x007c, 0, NULL);//s
ctrl_out(dev, 0x40, 1, 0x6040, 0x007d, 0, NULL);//src=64/128
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6007, 0x007c, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6000, 0x007d, 0, NULL);//cntr=0/256
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//s
ctrl_out(dev, 0x40, 1, 0x6001, 0x007c, 0, NULL);//s
ctrl_out(dev, 0x40, 1, 0x6080, 0x007d, 0, NULL);//sat=128/256
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//n
ctrl_out(dev, 0x40, 1, 0x6001, 0x0092, 0, NULL);//n
ctrl_out(dev, 0x40, 1, 0x60c0, 0x0093, 0, NULL);//net=0/32
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6002, 0x007c, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6000, 0x007d, 0, NULL);//coul=0/256
ctrl_out(dev, 0x40, 1, 0x6000, 0x00ff, 0, NULL);//g
ctrl_out(dev, 0x40, 1, 0x6008, 0x007c, 0, NULL);//g
ctrl_out(dev, 0x40, 1, 0x6020, 0x007d, 0, NULL);//gam=32/64

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)ms50);//droit

msleep(15);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x8004, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x6000, 0x8004, 1, (void*)ms28);
ctrl_out(dev, 0x40, 1, 0x60a8, 0x0004, 0, NULL);

msleep(15);
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x8004, 0, NULL);
ctrl_in(dev, 0xc0, 2, 0x6000, 0x8004, 1, (void*)msa8);
ctrl_out(dev, 0x40, 1, 0x60f8, 0x0004, 0, NULL);

ctrl_in(dev, 0xc0, 2, 0x0000, 0x0000, 1, (void*)ms50);//droit

ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x603e, 0x0024, 0, NULL);//c
ctrl_out(dev, 0x40, 1, 0x6034, 0x0025, 0, NULL);//cj=31/64

return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to modify the settings of the camera.
*
* This functions permits to modify the settings :
*   - brightness
*   - contrast
*   - white balance
*   - ...
*/
int dev_0503ms_camera_settings(struct usb_gl860 *dev)
{
int cj   =dev->vsettings.contrej;
int net  =dev->vsettings.net;
int sat  =dev->vsettings.saturation;
int coul =dev->vsettings.colour;
int src  =dev->vsettings.srclum;
int cntr =dev->vsettings.contrast;
int gam  =dev->vsettings.whiteness;
int lum  =dev->vsettings.brightness;

cj/=256*(256/64);
net/=256*(256/32);
sat/=256;
coul/=256;
src/=256*(256/128);
cntr/=256;
gam/=256*(256/64);
lum/=256;

STK_DEBUG("dev_0503ms_camera_settings [ini]n");

// Ordre du pilote écouté

if (cj!=excj)
{
if (cj<0) cj=0;
ctrl_out(dev, 0x40, 1, 0x6001      , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601f+cj   , 0x0024, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601f+cj-10, 0x0025, 0, NULL);
}

if (lum!=exlum)
{
if (lum<0) lum=0;
ctrl_out(dev, 0x40, 1, 0x6000    , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6009    , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+lum, 0x007d, 0, NULL);
exlum=lum;
}

if (src!=exsrc)
{
if (src<0) src=0;
ctrl_out(dev, 0x40, 1, 0x6000    , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6003    , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+src, 0x007d, 0, NULL);
exsrc=src;
}

if (cntr!=excntr)
{
if (cntr<0) cntr=0;
ctrl_out(dev, 0x40, 1, 0x6000     , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6007     , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+cntr, 0x007d, 0, NULL);
excntr=cntr;
}

if (sat!=exsat)
{
if (sat<0) sat=0;
ctrl_out(dev, 0x40, 1, 0x6000    , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001    , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+sat, 0x007d, 0, NULL);
exsat=sat;
}

if (net!=exnet)
{
if (net<0) net=0;
ctrl_out(dev, 0x40, 1, 0x6000    , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6001    , 0x0092, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x60c0+net, 0x0093, 0, NULL);
exnet=net;
}

if (coul!=excoul)
{
if (coul<0) coul=0;
ctrl_out(dev, 0x40, 1, 0x6000     , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6002     , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+coul, 0x007d, 0, NULL);
excoul=coul;
}

if (gam!=exgam)
{
if (gam<0) gam=0;
ctrl_out(dev, 0x40, 1, 0x6000    , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6008    , 0x007c, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000+gam, 0x007d, 0, NULL);
exgam=gam;
}

if (cj!=excj)
{
if (cj<0) cj=0;
ctrl_out(dev, 0x40, 1, 0x6001      , 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601f+cj   , 0x0024, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x601f+cj-10, 0x0025, 0, NULL);
excj=cj;
}

STK_DEBUG("dev_0503ms_camera_settings [fini]n");

return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to modify the settings of the camera.
*
* This functions permits to modify the settings :
*   - brightness
*   - contrast
*   - white balance
*   - ...
*
*/
int dev_0503ms_set_camera_quality(struct usb_gl860 *dev)
{
return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function permits to modify the settings of the camera.
*
* This functions permits to modify the frame rate per second.
*
*/
int dev_0503ms_set_camera_fps(struct usb_gl860 *dev)
{
return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function sets the device to start the stream.
*
* After the initialization of the device and the initialization of the video stream,
* this function permits to enable the stream.
*/
int dev_0503ms_start_stream(struct usb_gl860 *dev)
{

dev_0503ms_sensor_settings(dev);

return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief Reconfigure the camera before the stream.
*
* Before enabling the video stream, you have to reconfigure the device.
*/
int dev_0503ms_reconf_camera(struct usb_gl860 *dev)
{
switch (dev->resolution) {
case WCAM_640x480:
case WCAM_320x240:
case WCAM_160x120:
case WCAM_213x160:
case WCAM_128x96:
case WCAM_80x60:
dev->lealt=3;
dev->offset=2048;
dev->nbPaq=32;
break;

case WCAM_800x600:
dev->lealt=1;
dev->offset=3072;
dev->nbPaq=32;
break;

case WCAM_1280x960:
dev->lealt=1;
dev->offset=3072;
dev->nbPaq=32;
break;

case WCAM_1600x1200:
dev->lealt=1;
dev->offset=3072;
dev->nbPaq=32;
break;
}

return 0;
}


/**
* @param dev Device structure
*
* @returns 0 if all is OK
*
* @brief This function sets the device to stop the stream.
*
* You use the function start_stream to enable the video stream. So you
* have to use the function stop_strem to disable the video stream.
*/
int dev_0503ms_stop_stream(struct usb_gl860 *dev)
{
STK_DEBUG("dev_0503ms_stop_stream [ini]n");

ctrl_out(dev, 0x40, 5, 0x0000, 0x0000, 0, NULL);
msleep(20);
ctrl_out(dev, 0x40, 1, 0x006a, 0x000d, 0, NULL); //
ctrl_out(dev, 0x40, 1, 0x6001, 0x00ff, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6081, 0x0026, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0000, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0045, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6000, 0x0010, 0, NULL);
ctrl_out(dev, 0x40, 1, 0x6068, 0x000d, 0, NULL); //
msleep(50);
ctrl_out(dev, 0x40, 1, 0x0021, 0x0000, 0, NULL);

STK_DEBUG("dev_0503ms_stop_stream [fini]n");
return 0;
}

